import org.codehaus.groovy.grails.commons.ApplicationHolder

class OrderService {

    boolean transactional = true

    /**
     * 書籍を検索する。
     * 
     * 検索条件をもとに書籍を検索する。
     *   ・タイトル : 部分一致。大文字小文字区別しない。
     *   ・著者     : 部分一致。大文字小文字区別しない。
     *   ・ISBN 13  : 完全一致。ハイフンなし。
     *   ・出版社   : 部分一致。大文字小文字区別しない。
     * 
     * @param params パラメータ。以下のキーをからなるマップ。
     *            - title     : タイトル
     *            - author    : 著者
     *            - isbn13    : ISBN13
     *            - publisher : 出版社
     * @return 書籍の検索結果。リストで返す。
     *         検索結果が 0 件の場合、サイズ 0 のリストを返す。
     */
    def search(params) {

        assert params != null

        // 検索条件をもとに書籍を検索
        return Book.createCriteria().list() {
            // タイトル : 部分一致
            if (params.title) ilike("title", "%${params.title}%")

            // 著者 : 部分一致
            if (params.author) ilike("author", "%${params.author}%")

            // ISBN13 : 完全一致
            if (params.isbn13) eq("isbn13", "${params.isbn13}")

            // 出版社 : 部分一致
            if (params.publisher) {
                publisher {
                    ilike("name", "%${params.publisher}%")
                }
            }
        }
    }

    /**
     * 書籍をカートに入れる。
     * 
     * @param params パラメータ。以下のキーをからなるマップ。
     *            - id : 書籍のインスタンスの ID
     * @param order 注文
     */
    def putIntoCart(params, order) {

        assert params != null
        assert order != null

        // 書籍インスタンスと同じ id を持つ注文明細を検索
        def orderDetail = order.orderDetails.find {it.book.id == Long.valueOf(params.id)}

        // 同じ書籍が注文に入っている場合、書籍の冊数を 1 増やす
        // 同じ書籍が注文に入っていない場合、注文明細を設定
        if (orderDetail) {
            orderDetail.quantity++
        } else {
            // 書籍を取得
            def book = Book.get(params.id)

            // 注文明細を追加
            orderDetail = new OrderDetail(quantity : 1, book : book)
            order.addToOrderDetails(orderDetail)
        }
    }

    /**
     * カート内容を編集する。
     * 
     * @param params パラメータ。以下のキーをからなるマップ。
     *            - quantity$[書籍インスタンスの ID] : [書籍インスタンスの ID]が対応する書籍の冊数
     * @param order 注文
     * @return true:編集成功。false:編集失敗。
     */
    def editCart(params, order) {

        assert params != null
        assert order != null

        // 冊数の形式チェック
        def invalid = order.orderDetails.any {
            // 冊数を取得
            def quantity = params["quantity\$${it.book.id}"]

            // 冊数が整数かチェック
            !(quantity ==~ /[0-9]+/)
        }
        if (invalid) {
            return false
        }

        // 冊数を注文明細に設定し、削除する注文明細を取得
        def removeList = order.orderDetails.findAll {
            // 冊数を取得
            def quantity = params["quantity\$${it.book.id}"]

            // 冊数を設定
            it.quantity = new Integer(quantity)

            // 冊数が 0 の場合、注文明細を削除用リストに追加
            it.quantity == 0
        }

        // 削除用リストに追加された注文明細を注文から削除
        removeList.each {
            order.removeFromOrderDetails(it)
        }

        return true
    }

    /**
     * カート内容から合計金額を計算する。
     * 
     * @param order 注文
     * @return 合計金額
     */
    def calcTotalPrice(order) {

        assert order != null

        // 注文から合計金額を取得
        def totalPrice = order.getTotalPrice()

        // 合計金額が一定額以上の場合、割引を行う
        def config = ApplicationHolder.application.config
        def applyPrice = config.order.discount.applyPrice
        if ( totalPrice >= applyPrice ) {
            def rate = config.order.discount.rate
            totalPrice = (long)( totalPrice * ( 1 - rate ) )
        }

        return totalPrice
    }

}
