class OrderServiceTests extends GroovyTestCase {

    def orderService
    def bookList

    protected void setUp() {
        bookList = []
        def publisher = new Publisher(name:"翔泳社")
        assertNotNull publisher.save()
        def book = new Book(title:"オブジェクト指向入門 第 2 版 原則・コンセプト", author:"Bertrand Meyer", price:7560, releaseDate:new Date(), isbn13:"9784798111117", imageUrl:"")
        book.publisher = publisher
        assertNotNull book.save()
        bookList << book
        book = new Book(title:"UML モデリングのエッセンス 第 3 版 標準オブジェクトモデリング言語入門", author:"Martin Fowler", price:2520, releaseDate:new Date(), isbn13:"9784798107950", imageUrl:"")
        book.publisher = publisher
        assertNotNull book.save()
        bookList << book
    }

    /**
     * 無条件で書籍を検索。マッチする書籍が 2 冊あり。
     */
    void testSearchByNoConditionAndTwoData() {
        def params = [:]
        def searchResult = orderService.search(params)

        assertEquals 2, searchResult.size()
        assertNotNull searchResult.find({it.title == "オブジェクト指向入門 第 2 版 原則・コンセプト"})
        assertNotNull searchResult.find({it.title == "UML モデリングのエッセンス 第 3 版 標準オブジェクトモデリング言語入門"})
    }

    /**
     * タイトルを条件に書籍を検索。マッチする書籍が 1 冊あり。
     */
    void testSearchByTitleMatch() {
        def params = [title:"オブジェクト指向入門"]
        def searchResult = orderService.search(params)

        assertEquals 1, searchResult.size()
        assertNotNull searchResult.find({it.title == "オブジェクト指向入門 第 2 版 原則・コンセプト"})
    }

    /**
     * タイトルを条件に書籍を検索。マッチする書籍はなし。
     */
    void testSearchByTitleNotMatch() {
        def params = [title:"アスペクト指向入門"]
        def searchResult = orderService.search(params)

        assertEquals 0, searchResult.size()
    }

    /**
     * 著者を条件に書籍を検索。マッチする書籍が 1 冊あり。
     */
    void testSearchByAuthorMatch() {
        def params = [author:"Meyer"]
        def searchResult = orderService.search(params)

        assertEquals 1, searchResult.size()
        assertNotNull searchResult.find({it.title == "オブジェクト指向入門 第 2 版 原則・コンセプト"})
    }

    /**
     * 著者を条件に書籍を検索。マッチする書籍はなし。
     */
    void testSearchByAuthorNoMatch() {
        def params = [author:"Stevens"]
        def searchResult = orderService.search(params)

        assertEquals 0, searchResult.size()
    }

    /**
     * ISBN13 を条件に書籍を検索。マッチする書籍が 1 冊あり。
     */
    void testSearchByIsbn13Match() {
        def params = [isbn13:"9784798107950"]
        def searchResult = orderService.search(params)

        assertEquals 1, searchResult.size()
        assertNotNull searchResult.find({it.title == "UML モデリングのエッセンス 第 3 版 標準オブジェクトモデリング言語入門"})
    }

    /**
     * ISBN13 を条件に書籍を検索。マッチする書籍はなし。
     */
    void testSearchByIsbn13NoMatch() {
        def params = [isbn13:"9784798107951"]
        def searchResult = orderService.search(params)

        assertEquals 0, searchResult.size()
    }

    /**
     * 出版社を条件に書籍を検索。マッチする書籍が 2 冊あり。
     */
    void testSearchByPublisherMatch() {
        def params = [publisher:"翔泳社"]
        def searchResult = orderService.search(params)

        assertEquals 2, searchResult.size()
        assertNotNull searchResult.find({it.title == "オブジェクト指向入門 第 2 版 原則・コンセプト"})
        assertNotNull searchResult.find({it.title == "UML モデリングのエッセンス 第 3 版 標準オブジェクトモデリング言語入門"})
    }

    /**
     * 出版社を条件に書籍を検索。マッチする書籍はなし。
     */
    void testSearchByPublisherNoMatch() {
        def params = [publisher:"翔英社"]
        def searchResult = orderService.search(params)

        assertEquals 0, searchResult.size()
    }

    /**
     * タイトルと著者と ISBN13 と出版社を条件に書籍を検索。マッチする書籍が 1 冊あり。
     */
    void testSearchByTitleAndAuthorAndIsbn13AndPublisherMatch() {
        def params = [title:"エッセンス", author:"Fowler", isbn13:"9784798107950", publisher:"翔泳社"]
        def searchResult = orderService.search(params)

        assertEquals 1, searchResult.size()
        assertNotNull searchResult.find({it.title == "UML モデリングのエッセンス 第 3 版 標準オブジェクトモデリング言語入門"})
    }

    /**
     * タイトルと著者と ISBN13 と出版社を条件に書籍を検索。マッチする書籍はなし。
     */
    void testSearchByTitleAndAuthorAndIsbn13AndPublisherNoMatch() {
        def params = [title:"エッセンス", author:"Fowler", isbn13:"9784798107951", publisher:"翔泳社"]
        def searchResult = orderService.search(params)

        assertEquals 0, searchResult.size()
    }

    /**
     * 空のカートに書籍を 1 冊入れる。
     */
    void testPutIntoCartFirst() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id
    }

    /**
     * カートに書籍が 1 冊入っている状態で、同じ書籍を 1 冊入れる。
     */
    void testPutIntoCartSecondSameBook() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        orderDetail = order.orderDetails.find { it }
        assertEquals 2, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id
    }

    /**
     * カートに書籍が 1 冊入っている状態で、違う書籍を 1 冊入れる。
     */
    void testPutIntoCartSecondDifferentBook() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        params.id = bookList[1].id
        orderService.putIntoCart(params, order)

        assertEquals 2, order.orderDetails.size()
        order.orderDetails.each {
            assertEquals 1, it.quantity
            assertTrue it.book.id == bookList[0].id || it.book.id == bookList[1].id
        }
    }

    /**
     * カートに書籍が入っていない状態で、カート内容を編集する。
     */
    void testEditCartNoOrderDetail() {
        def params = [:]
        def order = new Order()
        assertTrue orderService.editCart(params, order)
    }

    /**
     * カートに書籍が 1 冊入っている状態で、冊数を 3 に変更する。
     */
    void testEditCartQuantityTo3() {
        def params = [id:bookList[0].id]
        def order = new Order()

        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        params = [:]
        params["quantity\$${bookList[0].id}"] = '3'
        assertTrue orderService.editCart(params, order)

        assertEquals 1, order.orderDetails.size()
        orderDetail = order.orderDetails.find { it }
        assertEquals 3, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id
    }

    /**
     * カートに書籍が 1 冊入っている状態で、冊数を 0 に変更する。
     */
    void testEditCartQuantityTo0() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        params = [:]
        params["quantity\$${bookList[0].id}"] = '0'
        assertTrue orderService.editCart(params, order)
        assertEquals 0, order.orderDetails.size()
    }

    /**
     * カートに異なる書籍が 1 冊ずつ入っている状態で、冊数をそれぞれ 3 に変更する。
     */
    void testEditCartQuantityTo3Many() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        params.id = bookList[1].id
        orderService.putIntoCart(params, order)

        assertEquals 2, order.orderDetails.size()
        order.orderDetails.each {
            assertEquals 1, it.quantity
            assertTrue it.book.id == bookList[0].id || it.book.id == bookList[1].id
        }

        params = [:]
        params["quantity\$${bookList[0].id}"] = '3'
        params["quantity\$${bookList[1].id}"] = '3'
        assertTrue orderService.editCart(params, order)

        assertEquals 2, order.orderDetails.size()
        order.orderDetails.each {
            assertEquals 3, it.quantity
            assertTrue it.book.id == bookList[0].id || it.book.id == bookList[1].id
        }
    }

    /**
     * カートに異なる書籍が 1 冊ずつ入っている状態で、片方だけ冊数を 0 に変更する。
     */
    void testEditCartQuantityTo3or0() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        params.id = bookList[1].id
        orderService.putIntoCart(params, order)

        assertEquals 2, order.orderDetails.size()
        order.orderDetails.each {
            assertEquals 1, it.quantity
            assertTrue it.book.id == bookList[0].id || it.book.id == bookList[1].id
        }

        params = [:]
        params["quantity\$${bookList[0].id}"] = '0'
        params["quantity\$${bookList[1].id}"] = '3'
        assertTrue orderService.editCart(params, order)

        assertEquals 1, order.orderDetails.size()
        orderDetail = order.orderDetails.find { it }
        assertEquals 3, orderDetail.quantity
        assertEquals bookList[1].id, orderDetail.book.id
    }

    /**
     * カートに書籍が 1 冊入っている状態で、冊数を X に変更する。
     */
    void testEditCartQuantityToX() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        params = [:]
        params["quantity\$${bookList[0].id}"] = 'X'
        assertFalse orderService.editCart(params, order)

        assertEquals 1, order.orderDetails.size()
        orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id
    }

    /**
     * カートに異なる書籍が 1 冊ずつ入っている状態で、片方だけ冊数を X に変更する。
     */
    void testEditCartQuantityTo3orX() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        assertEquals 1, order.orderDetails.size()
        def orderDetail = order.orderDetails.find { it }
        assertEquals 1, orderDetail.quantity
        assertEquals bookList[0].id, orderDetail.book.id

        params.id = bookList[1].id
        orderService.putIntoCart(params, order)

        assertEquals 2, order.orderDetails.size()
        order.orderDetails.each {
            assertEquals 1, it.quantity
            assertTrue it.book.id == bookList[0].id || it.book.id == bookList[1].id
        }

        params = [:]
        params["quantity\$${bookList[0].id}"] = '3'
        params["quantity\$${bookList[1].id}"] = 'X'
        assertFalse orderService.editCart(params, order)

        assertEquals 2, order.orderDetails.size()
        order.orderDetails.each {
            assertEquals 1, it.quantity
            assertTrue it.book.id == bookList[0].id || it.book.id == bookList[1].id
        }
    }

    /**
     * カートに書籍が 1 冊入っている状態で合計金額を計算する。
     */
    void testCalcTotalPriceOneBookOneQuantity() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        def totalPrice = orderService.calcTotalPrice(order)
        assertEquals 7560, totalPrice
    }

    /**
     * カートに同じ書籍が 3 冊入っている状態で合計金額を計算する。
     */
    void testCalcTotalPriceOneBookThreeQuantity() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)
        orderService.putIntoCart(params, order)
        orderService.putIntoCart(params, order)

        def totalPrice = orderService.calcTotalPrice(order)
        assertEquals( ( 7560 * 3 ) * 0.9, totalPrice )
    }

    /**
     * カートに異なる書籍が 1 冊ずつ入っている状態で合計金額を計算する。
     */
    void testCalcTotalPriceTwoBookOneQuantity() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)

        params.id = bookList[1].id
        orderService.putIntoCart(params, order)

        def totalPrice = orderService.calcTotalPrice(order)
        assertEquals 7560 + 2520, totalPrice
    }

    /**
     * カートに異なる書籍が 3 冊ずつ入っている状態で合計金額を計算する。
     */
    void testCalcTotalPriceTwoBookThreeQuantity() {
        def params = [id:bookList[0].id]
        def order = new Order()
        orderService.putIntoCart(params, order)
        orderService.putIntoCart(params, order)
        orderService.putIntoCart(params, order)

        params.id = bookList[1].id
        orderService.putIntoCart(params, order)
        orderService.putIntoCart(params, order)
        orderService.putIntoCart(params, order)

        def totalPrice = orderService.calcTotalPrice(order)
        assertEquals( ( 7560 * 3 + 2520 * 3 ) * 0.9, totalPrice )
    }
}
